local frac = FRACUNIT

addHook("ThinkFrame", function()
	for p in players.iterate
		if not (p and p.mo and p.mo.valid and p.mo.skin == "knuckles") then return end
		// spindash up wall (walldash?)
		if p.climbing >= 1 // startup
		and (p.cmd.buttons & BT_CUSTOM1)
		and p.walldashAvailable
			p.mo.state = S_PLAY_DROP
			p.walldash = true
		end
		
		if p.walldash // effects
			P_SpawnSkidDust(p, 15*frac)
			p.pflags = $1|PF_FULLSTASIS
		end
		
		if p.walldash // charge
			p.walltime = $ + 1
		end
		
		if p.walltime > 40 // cap
			p.walltime = 40
		end
		
		if (p.walltime % 4 == 1) // spin sound
		and p.walltime < 40
			S_StopSoundByID(p.mo, sfx_spndsh)
			S_StartSound(p.mo, sfx_spndsh)
		end
		
		if p.walldash // launch
		and not (p.cmd.buttons & BT_CUSTOM1)
		and p.walldashAvailable
			if p.powers[pw_super]
				P_SetObjectMomZ(p.mo, (p.walltime*frac)/3+(frac*20), false)
			else
				P_SetObjectMomZ(p.mo, (p.walltime*frac)/3+(frac*15), false)
			end
			S_StartSound(p.mo, sfx_zoom)
			p.pflags = $1 & ~PF_FULLSTASIS
			p.walldash = false
			p.climbing = 0
		end
		
		if P_IsObjectOnGround(p.mo) // reset
		or not (p.cmd.buttons & BT_CUSTOM1)
			p.walldash = false
			p.walltime = 0
		end
		
		if p.c1 // cancel glide
		and (p.mo.state == S_PLAY_GLIDE or p.mo.state == S_PLAY_SWIM)
			P_SetObjectMomZ(p.mo, 3*frac, false)
			S_StartSound(nil, sfx_rerol, p)
			p.mo.state = S_PLAY_ROLL
			p.pflags = $1|PF_THOKKED|PF_JUMPED
			p.pflags = $ & ~PF_GLIDING
		end
		
		// drill
		if p.c2
		and not p.drilling
		and p.playerstate ~= PST_DEAD
		and not P_IsObjectOnGround(p.mo)
		and (p.pflags & PF_JUMPED)
		and p.drillAvailable
			P_SetObjectMomZ(p.mo, -30*frac, false)
			S_StartSound(nil, sfx_zoom, p)
			p.pflags = $1|PF_THOKKED
			p.drilling = true
		end
		
		if p.drilling // state and effects
			p.mo.state = S_PLAY_DRILL
			p.drilltime = $ + 1
			X_SpawnGhostTrail(p, 0, false)
		end
		
		if p.mo.state == S_PLAY_DRILL // reset state
		and not p.drilling
			p.mo.state = S_PLAY_FALL
		end
		
		if P_IsObjectOnGround(p.mo) // hit ground
		and p.drilling
		and p.drillAvailable
			if p.powers[pw_super]
				P_StartQuake(55*frac, 2)
				P_NukeEnemies(p.mo, p.mo, 600*frac)
				S_StartSound(nil, sfx_s3k49, p)
			else
				P_StartQuake(35*frac, 2)
				P_NukeEnemies(p.mo, p.mo, 400*frac)
				S_StartSound(nil, sfx_s3k4c, p)
			end
			p.drilling = false
			p.drilltime = 0
		end
		
		if (p.drilltime > 0 or p.drilling) // hit spring (doesnt work so i dont care at this point)
		and p.mo.state == S_PLAY_SPRING
			P_SetObjectMomZ(p.mo, (5*frac)+p.mo.momz, false)
			S_StopSoundByID(p.mo, sfx_spring)
			S_StartSound(nil, sfx_sprong, p)
			p.drilling = false
			p.drilltime = 0
		end
		
		if p.mo.momz > 1*frac
		and p.drilling
			p.drilling = false
			p.drilltime = 0
		end
		
		if P_IsObjectOnGround(p.mo)
			p.drilling = false
			p.drilltime = 0
		end
		
		// dash (while gliding)
		if p.c2
		and (p.mo.state == S_PLAY_GLIDE or p.mo.state == S_PLAY_SWIM)
		and not p.dash
		and p.glidedashAvailable
			if p.powers[pw_super]
				P_StartQuake(35*frac, 2)
				P_InstaThrust(p.mo, p.mo.angle, p.speed + 25*frac)
				P_NukeEnemies(p.mo, p.mo, 400*frac)
				S_StartSound(nil, sfx_peel, p)
			else
				P_InstaThrust(p.mo, p.mo.angle, p.speed + 15*frac)
				S_StartSound(nil, sfx_zoom, p)
			end
			p.dash = true
		end
		
		if not (p.pflags & PF_GLIDING) // reset values
		and p.dash
			p.dash = false
		end
		
		if p.dash
			X_SpawnGhostTrail(p, 0, false)
		end
	end
end)

// check and crumble stuff by Tatsuru in srb2 server
local function CheckAndCrumble(mo, sec)
	for fof in sec.ffloors()
		if not (fof.flags & FF_EXISTS) continue end -- Does it exist?
		if not (fof.flags & FF_BUSTUP) continue end -- Is it bustable?

		if mo.z + mo.height < fof.bottomheight continue end -- Are we too low?
		if mo.z > fof.topheight + frac continue end -- Are we too high?

		-- Check for whatever else you may want to    

		EV_CrumbleChain(fof) -- Crumble
	end
end

addHook("PlayerThink", function(p)
	if not p.mo return end
	if p.mo.skin != "knuckles" return end

	if p.mo.state == S_PLAY_DRILL
		CheckAndCrumble(p.mo, p.mo.subsector.sector)
	end
end)

addHook("MobjLineCollide", function(mo, line)
	if not mo.player return end
	if mo.skin != "knuckles" return end

	for _, sec in ipairs({line.frontsector, line.backsector})
		if mo.state == S_PLAY_DRILL
			CheckAndCrumble(mo, sec)
		end
	end
end, MT_PLAYER)

addHook("TouchSpecial", function(mobj, pmo)
	if pmo.skin ~= "knuckles" return end
	
    if pmo.state == S_PLAY_DRILL and (mobj.flags & MF_ENEMY)
        P_KillMobj(mobj, pmo, pmo)
		P_SetObjectMomZ(pmo, 10*frac, false)
		pmo.player.drilling = false
    end
	
	if pmo.state == S_PLAY_DRILL and (mobj.flags & MF_MONITOR)
        P_KillMobj(mobj, pmo, pmo)
		P_SetObjectMomZ(pmo, -20*frac, false)
		print("it did it")
		pmo.player.drilling = false
    end
end)

